/* CSD: Cornerstone disassembler

    Copyright (C) 2014  John Elliott <jce@seasip.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

package info.seasip.csd;


import java.io.*;

/** Represents the table of global variables. Loaded from the .MME file.
 *
 * @author      John Elliott <jce@seasip.demon.co.uk>
 * @version     1.0
 * @since       2014-04-12
 */
public class GlobalIndex extends WordChunk
{
	/** Number of global variables */
	private int globalCount;
	/** Number of module-global variables in each module */
	private int mgc[];
	/** Values of module-global variables by module */ 
	private ModuleGlobals moduleGlobals[];
	/** Position of index in file. */
	private long base;	


	/** Constructor.
	 * @param rf The .MME file.
	 */
	GlobalIndex(RandomAccessFile rf)
	{
		super(rf);
	}

	/** Load the index and the module globals. 
	 * @param modCount count of modules.
	 * @throws IOException on file access error.
	 */
	public void load(int modCount) throws IOException
	{
		int n;

		base = file.getFilePointer();
		// Count of globals
		globalCount = read();
		moduleGlobals = new ModuleGlobals[modCount];
		mgc           = new int[modCount];
		// Count of globals in each module
		for (n = 0; n < modCount; n++)	
		{
			mgc[n] = read();
		}
		// Global globals
		read(globalCount);
		// Module globals
		for (n = 0; n < modCount; n++)	
		{
			moduleGlobals[n] = new ModuleGlobals(file, mgc[n], n);
			moduleGlobals[n].load()	;
		}	
	}

	/** Accessor.
	 * @return Count of program-wide globals. */
	public int getGlobalCount() 
	{ 
		return globalCount;
	}
	
	/** Get the module globals for a given module.
         * @param module The module ID (0-based).
	 * @return The module globals for the specified module. */
	public ModuleGlobals getModuleGlobals(int module)
	{
		return moduleGlobals[module];	
	}


	/** Dump the contents of the globals index.
	 * @param stream The stream to output to.
	 */
	public void showContents(PrintStream stream)
	{
		long pos;

		stream.println(";---------------------------------------");
		stream.println("; Globals index:");
		stream.println(";---------------------------------------");
		stream.printf ("; %04x: Globals count = %d\n", base, 
					globalCount);
		for (int n = 0; n < mgc.length; n++)
		{
			stream.printf ("; %04x: Module %d globals count = %d\n",
				base + 2 + 2 * n, 
				n + 1, mgc[n]);	
		}
		pos = base + 2 + 2 * mgc.length;
		for (int n = 0; n < globalCount; n++)
		{
			stream.printf ("; %04x: Global %d = 0x%04x\n", 
				pos + 2 * n, 
				n,
				words[n]);	

		}
		for (int n = 0; n < mgc.length; n++)
		{
			stream.println(n);
			moduleGlobals[n].showContents(stream);
		}
	}
}

