/* CSD: Cornerstone disassembler

    Copyright (C) 2014  John Elliott <jce@seasip.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

package info.seasip.csd;
import java.io.*;
import java.util.Vector;
import java.util.HashMap;

/** Symbol table: A collection of Symbols.
 *
 * @author      John Elliott <jce@seasip.demon.co.uk>
 * @version     1.0
 * @since       2014-04-12
 */
public class SymbolTable
{
	/** The Symbols in this symbol table. */
	HashMap<Integer, Symbol> map = new HashMap<Integer, Symbol>();

	/** Look up a specified symbol.
	 *
	 * @param module The module ID, 1-based. 0 may be used in future for
	 * 		addresses in RAM. 
	 * @param offset The offset within the module. 
	 * @return The corresponding Symbol if it is present, otherwise null.
	 */
	public Symbol lookup(int module, int offset)
	{
		int key = Symbol.getKey(module, offset);
		
		return map.get(key);
	}

	/** Add a new symbol to the map.
	 *
	 * @param type The type of symbol, see Symbol.type
	 * @param name The text of the symbol.
	 * @param module The module ID, 1-based.
	 * @param offset The offset within the module.
	 */
	public void add(char type, String name, int module, int offset)
	{
		map.put(Symbol.getKey(module, offset), 
			new Symbol(type, name, module, offset));
	}

	/** Disgorge all symbols in a specified module.
	 *
	 * @param module The module ID, 1-based.
	 * @param type The type of symbol, see Symbol.type. Pass (char)0 to
	 * 		return symbols of all types.
	 * @return An array containing the requested Symbols.
	 */
	public Symbol[] moduleSymbols(int module, char type)
	{
		// Accumulate results in a Vector.
		Vector<Symbol> results = new Vector<Symbol>();

		for (Symbol value : map.values())
		{
			if (value.module != module) continue;
			if (type == 0 || value.type == type) 
				results.add(value);
		}
		Symbol[] r = new Symbol[results.size()];
		results.toArray(r);
		return r;
	}

	/** Diagnostic dump.
	 *
	 * @param os The stream to dump the contents to.
	 */
	public void dump(java.io.PrintStream os)
	{
		for (Symbol value : map.values())
		{
			os.printf("type=%c offset=%04x module=%d\n",
				value.type, value.offset, value.module);	
		}	
	}


	// Parse a single Symbol from the file.
	private static Symbol readSymbol(StreamTokenizer st) throws IOException
	{
		Symbol s = new Symbol();

		st.nextToken();
		if (st.ttype == StreamTokenizer.TT_EOF) return null;				
		if (st.ttype != StreamTokenizer.TT_WORD)
		{
			System.err.println("Can't parse symbol type: " + st);	
			System.exit(1);
		}
		s.type = st.sval.charAt(0);
	
		st.nextToken();
		if (st.ttype != StreamTokenizer.TT_WORD)
		{
			System.err.println("Can't parse label: " + st);	
			System.exit(1);
		}
		s.name = st.sval;
	
		st.nextToken();
		if (st.ttype != StreamTokenizer.TT_NUMBER) 
		{
			System.err.println("Can't parse module: " + st);	
			System.exit(1);
		}
		s.module = (int)st.nval;

		st.nextToken();
		if (st.ttype == StreamTokenizer.TT_EOL) return s;
		if (st.ttype != StreamTokenizer.TT_NUMBER)
		{
			System.err.println("Can't parse offset: " + st);	
			System.exit(1);
		}
		s.offset = (int)st.nval;
		
		st.nextToken();
		if (st.ttype == StreamTokenizer.TT_EOL) return s;
		if (st.ttype == StreamTokenizer.TT_EOF) return s;
		System.err.println("Can't parse symbol");	
		System.exit(1);
		return null;
	}
	
	/** Load symbols from a file.
	 *
	 * The field order in the file is: type,label,module,offset.
	 *
	 * @param r The file from which symbols will be loaded.
	 * @exception IOException If the read fails.
	 */
	public void loadSymbols(Reader r) throws IOException
	{
		StreamTokenizer st = new StreamTokenizer(r);
		map = new HashMap<Integer, Symbol>();
		Symbol s;
		int n;

		st.commentChar(';');
		st.commentChar('!');
		st.commentChar('#');
		st.quoteChar('"');
		st.eolIsSignificant(true);
		st.slashSlashComments(true);
	
		while ((s = readSymbol(st)) != null) 
		{
			map.put(s.getKey(), s);
		}
	}
}

